
	module FSM(HR, HY, HG, FR, FY, FG, ST, TS, TL, C, reset, clock);
	output HR, HY, HG, FR, FY, FG, ST;
	input TS, TL, C, reset, clock;
	
	reg [6:1] state;
	reg ST;
	
	parameter highwaygreen = 6'b001100;
	parameter highwayyellow = 6'b010100;
	parameter farmroadgreen = 6'b100001;
	parameter farmroadyellow = 6'b100010;
	
	assign HR = !state[6];    // Negation required due to the
	assign HY = !state[5];    // negative I/O logic of the C-M240 card
	assign HG = !state[4];
	assign FR = !state[3];
	assign FY = !state[2];
	assign FG = !state[1];
	
	initial begin
		 state = highwaygreen;
		 ST = 0;
	end
	
	always @(posedge clock) begin
		 if (reset) begin
			  state = highwaygreen;
			  ST = 1;
		 end
		 else begin
			  ST = 0;
			  case (state)
					highwaygreen:
						 if (TL & C) begin
							  state = highwayyellow;
							  ST = 1;
						 end
					highwayyellow:
						 if (TS) begin
							  state = farmroadgreen;
							  ST = 1;
						 end
					farmroadgreen:
						 if (TL | !C) begin
							  state = farmroadyellow;
							  ST = 1;
						 end
					farmroadyellow:
						 if (TS) begin
							  state = highwaygreen;
							  ST = 1;
						 end
			  endcase
		 end
	end
	endmodule
	
	module Timer(TS, TL, ST, clock);
	output TS;
	output TL;
	input ST;
	input clock;
	reg [7:0] value;
	
	assign TS = (value > 4);       //  5 cycles after reset
	assign TL = (value > 19);      // 20 cycles after reset
	
	always @(posedge clock, posedge ST)
	  if(ST==1) value = 0;
	  else value = value + 1;
	endmodule
	
	module main(HR, HY, HG, FR, FY, FG, nReset, nC, clk_50M,C,reset);
	  output HR, HY, HG, FR, FY, FG, C, reset;
	  input nReset, nC, clk_50M;
	  reg clock;
	  reg [24:0] count = 0;
	  assign C = !nC;              // Negation required due to the
	  assign reset = !nReset;      // negative I/O logic of the C-M240 card
	
	  Timer part1(TS, TL, ST, clock);
		 FSM part2(HR, HY, HG, FR, FY, FG, ST, TS, TL, C, reset, clock); 
	  
	  //--- Clock signal generation ---------------------
	  always @ (posedge clk_50M)    
		 begin
			if (count == 25000000)
			  begin                  // our clock is of 50,000,000 Hz
											 // We now use the count counter 25,000,000 cycles
				 clock <= ~clock;     // so that we get a signal with 1 s period (1 Hz).
				 count <= 0;          // clear the counter.
			  end
			else
			  count <= count + 1;    // counter from the increase.
		 end
	
	endmodule
